@@ -253,6 +253,25 @@ class ZipBuilder(object):
             0xFFFFFFFF if use_zip64 else self.ctx.relative_offset,
         ), self.ctx.path, extra, self.ctx.comment)
 
+    def add_buf(self, path: AnyStr, buf: Union[bytes, bytearray, memoryview], utc_time: Optional[float] = None, compression=COMPRESSION_STORED, comment="") -> Generator[bytes, None, None]:
+        """Adds io and returns Generator of bytes object."""
+        # Create file context.
+        self.ctx = self._new_file_ctx(
+            path, None, utc_time, compression, comment
+        )
+
+        # Yield file's header and content.
+        try:
+            yield self._write_local_file()
+
+            for buf in compress_buf(self.ctx.compressor, self.ctx.compressor_ctx, buf, len(self.buffer)):
+                yield self._write(buf)
+
+            yield self._write_data_descriptor()
+        finally:
+            self._set_header()
+            self._clear_ctx()
+
     def add_io(self, path: AnyStr, io: Union[BufferedIOBase, RawIOBase], utc_time: Optional[float] = None, compression=COMPRESSION_STORED, comment="") -> Generator[bytes, None, None]:
         """Adds io and returns Generator of bytes object."""
         with io:
@@ -368,6 +387,7 @@ class ZipBuilder(object):
 
         offset = self.offset
         time, date = dos_time(utc_time)
+        use_zip64 = offset >= INT32_MAX
 
         # LocalFile
         buf = self._write(pack_header_with_data(HEADER_LOCAL_FILE, LocalFile(
@@ -383,6 +403,15 @@ class ZipBuilder(object):
             0,  # Len extra
         ), path_bytes))
 
+        # Extended information for zip64
+        extra = pack_header(TAG_EXTENDED_INFORMATION64, ExtendedInformation64(
+            SIZE_EXTENDED_INFORMATION,  # Size of extended information.
+            0,
+            0,
+            offset,
+            0,  # Disk start number
+        )) if use_zip64 else b""
+
         # CentralDirectory
         self.headers[path_bytes] = pack_header_with_data(HEADER_CENTRAL_DIRECTORY, CentralDirectory(
             self.version_extract,
@@ -393,16 +422,16 @@ class ZipBuilder(object):
             time,
             date,
             0,  # Crc32
-            0,  # Compressed len
-            0,  # Uncompressed len
+            0xFFFFFFFF if use_zip64 else 0,  # Compressed len
+            0xFFFFFFFF if use_zip64 else 0,  # Uncompressed len
             len(path_bytes),
-            0,  # Extra len
+            len(extra),  # Extra len
             len(comment_bytes),
             0,  # Disks tart
             0,  # Internal attr
             DEFAULT_EXTERNAL_DIR_ATTR,  # External attr
-            offset,
-        ), path_bytes, b"", comment_bytes,)
+            0xFFFFFFFF if use_zip64 else offset,
+        ), path_bytes, extra, comment_bytes,)
 
         return buf
 
